<?php
declare(strict_types=1);

namespace Catequil\Barcode\Service;

use Catequil\Barcode\Resolver\ResolverInterface;
use Catequil\Barcode\Resolver\ResolverPluginManager;
use Catequil\Core\Exception\NotFoundException;
use InvalidArgumentException;
use Picqer\Barcode\BarcodeGeneratorPNG;
use function sprintf;

/**
 * Class BarcodeService
 *
 * @package Catequil\Barcode\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-barcode
 */
final class BarcodeService implements BarcodeServiceInterface
{
    private ResolverPluginManager $resolvers;

    /**
     * @param ResolverPluginManager $resolvers
     */
    public function __construct(ResolverPluginManager $resolvers)
    {
        $this->resolvers = $resolvers;
    }

    //<editor-fold desc="Resolver">

    public function combine(string $type, int|string $value): string
    {
        $resolver = $this->getResolver($type);
        $combined = $resolver->combine($value);

        if ($combined === null) {
            throw NotFoundException::create();
        }

        return $combined;
    }

    public function resolve(string $type, string $barcode): int|string
    {
        $resolver = $this->getResolver($type);
        $resolved = $resolver->resolve($barcode);

        if ($resolved === null) {
            throw NotFoundException::create(
                sprintf(
                    'Nepostojeći barkod: %s',
                    $barcode
                )
            );
        }

        return $resolved;
    }

    private function getResolver(string $type): ResolverInterface
    {
        if (!$this->resolvers->has($type)) {
            throw new InvalidArgumentException(
                sprintf('Invalid barcode type "%s".', $type)
            );
        }

        return $this->resolvers->get($type);
    }

    //</editor-fold>
    //<editor-fold desc="Generator">

    public function generate(string $value): string
    {
        $generator = new BarcodeGeneratorPNG();

        return $generator->getBarcode(
            $value,
            $generator::TYPE_CODE_128,
        );
    }

    //</editor-fold>
}
